################################################################
#
#   Bell Runner - Level
#
################################################################

from albow.resource import get_image
from albow.controls import Label, Grid, Row, Column, Button
from albow.dialogs import Dialog
from player import Player

cell_width = 16
cell_height = 16

cells_wide = 50
cells_high = 36

#------------------------------------------------------------------------

class Blob(object):
	pass

#------------------------------------------------------------------------

class Item(object):
	#  rect         Rect
	#  image_name   string
	
	deletable = True
	editable = False
	filename_suffix = ".png"
	
	def __init__(self, rect, image_name):
		self.rect = rect
		self.image_name = image_name
	
	def reset(self):
		pass
	
	def offset(self, delta):
		dx, dy = delta
		self.rect.move_ip(dx, dy)
	
	def draw(self, surf):
		name = self.current_image_name()
		image = get_item_image(self.__class__, name)
		surf.blit(image, self.rect)
	
	def current_image_name(self):
		return self.image_name
	
	def edit(self):
		field_specs = []
		self.create_edit_fields(field_specs)
		if field_specs:
			fields = Blob()
			controls = []
			for name, title, field in field_specs:
				setattr(fields, name, field)
				controls.append((Label(title), field))
			grid = Grid(controls)
			dlog = Dialog(client = grid, responses = ["OK", "Cancel"])
			self.load_edit_fields(fields)
			resp = dlog.present()
			if resp == "OK":
				self.unload_edit_fields(fields)
				return True
		return False
	
	def create_edit_fields(self, field_specs):
		pass
	
	def load_edit_fields(self, fields):
		pass
	
	def unload_edit_fields(self, fields):
		pass
	
	def bell_rung(self, pitch):
		pass
	
	def actor_nearby(self, level, actor):
		pass
	
	def level_completed(self):
		game.level_completed = True

#------------------------------------------------------------------------

class Decoration(Item):
	resource_prefix = "decorations"
	layer_name = 'background'


class Platform(Item):
	resource_prefix = "platforms"
	layer_name = 'platforms'

#------------------------------------------------------------------------

class Level(object):
	#  background    [Decoration]
	#  gadgets       [Item]
	#  platforms     [Platform]
	#  bells         [Bell]
	#  actors        [Player]
	#  player        Player
	#  is_completed  boolean

	def __init__(self):
		self.player = Player((512, 512))
		self.background = []
		self.gadgets = []
		self.platforms = []
		self.bells = []
		self.actors = [self.player]
	
	def reset(self):
		for layer in self.layers():
			for item in layer:
				item.reset()
		self.is_completed = False

	def add_item(self, item):
		getattr(self, item.layer_name).append(item)
	
	def draw(self, surf):
		for layer in self.layers():
			for item in layer:
				item.draw(surf)
	
	def layers(self):
		return [self.background, self.gadgets, self.platforms, self.bells, self.actors]
	
	def find_layer(self, item):
		return getattr(self, item.layer_name)
	
	def begin_frame(self):
		for item in self.actors:
			self.actor_begin_frame(item)
		for item in self.bells:
			item.begin_frame(self)
	
	def actor_begin_frame(self, actor):
		level_bottom = cells_high * cell_height
		g = actor.get_grab_position()
		actor.nearby_rope = self.rope_near_point(g)
		p = actor.position
		platform = self.item_at_point('platforms', p)
		if platform:
			actor.supported = True
			actor.support_height = platform.rect.top
		elif p[1] >= level_bottom:
			actor.supported = True
			actor.support_height = level_bottom
		else:
			actor.supported = False
		for gadget in self.gadgets_near_actor(actor):
			gadget.actor_nearby(self, actor)
		actor.begin_frame(self)
	
	def item_at_point(self, layer_name, p):
		for item in getattr(self, layer_name):
			if item.rect.collidepoint(p):
				return item
	
	def rope_near_point(self, p):
		#print "Level: grab point =", p ###
		for item in self.bells:
			r = item.get_rope_rect()
			#print "Level: rope rect =", r ###
			r.inflate_ip((40, 0))
			if r.collidepoint(p):
				#print "Level: Nearby rope =", item ###
				return item
	
	def gadgets_near_actor(self, actor):
		r = actor.rect
		for item in self.gadgets:
			if item.rect.colliderect(r):
				yield item 
	
	def bell_rung(self, pitch):
		for item in self.gadgets:
			item.bell_rung(pitch)
	
	def level_completed(self):
		self.is_completed = True

#------------------------------------------------------------------------

def get_item_image(kind, name):
	return get_image("%s/%s" % (kind.resource_prefix, name))

def round_to_grid(p):
	cw = cell_width
	ch = cell_height
	x, y = p
	x = (x + cw // 2) // cw * cw
	y = (y + ch // 2) // ch * ch
	return x, y
